/*
 *                 IFS Research & Development
 *
 *  This program is protected by copyright law and by international
 *  conventions. All licensing, renting, lending or copying (including
 *  for private use), and all other use of the program, which is not
 *  expressively permitted by IFS Research & Development (IFS), is a
 *  violation of the rights of IFS. Such violations will be reported to the
 *  appropriate authorities.
 *
 *  VIOLATIONS OF ANY COPYRIGHT IS PUNISHABLE BY LAW AND CAN LEAD
 *  TO UP TO TWO YEARS OF IMPRISONMENT AND LIABILITY TO PAY DAMAGES.
 * ----------------------------------------------------------------------------
 * File        : MyListConnectReader.java
 * Description :
 * Notes       :
 * ----------------------------------------------------------------------------
 * Modified    :
 *    2016-Feb-18  Jacek P  Created.
 * ----------------------------------------------------------------------------
 */

package ifs.fndint.connectreader;

import ifs.fnd.connect.readers.ConnectReader;
import ifs.fnd.util.IoUtil;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.*;

public class MyListConnectReader extends ConnectReader<MyListConnectReaderConfig> {

    private transient String fileName;

    /**
     * Initialization of resources used by the reader. The reader implements the
     * AutoCloseable interface, so all resources initiated/open here have to be
     * released/closed in the nativeClose() method.
     *
     * @throws ConnectReader.ReaderFailureException
     */
    @Override
    public void nativeInitReader() throws ReaderFailureException {
        if (log.debug) log.debug("Initializing My List Connect Reader [&1]...", config.instance);
        try {
            IoUtil.mkdirs(new File(config.inDir));
        } catch (IOException e) {
            throw new PermanentFailureException(e, "Error while creating input directory [&1]", config.inDir);
        }
    }

    /**
     * Initialization of resources used per message basis. Resources
     * initiated/open here have to be released/closed in the nativeCloseMessage()
     * method.
     *
     * @param messageId
     * @throws ConnectReader.ReaderFailureException
     */
    @Override
    public void nativeInitMessage(String messageId) throws ReaderFailureException {
        // decode message ID; in our case it is just the file name
        if (log.debug) log.debug("Message ID/file name: [&1]", messageId);
        fileName = messageId;
    }

    @Override
    public List<String> nativeList() throws ReaderFailureException {
        if (log.debug) log.debug("Listing directory [&1]...", config.inDir);
        SortedSet<String> set = new TreeSet<>();
        File inDir = new File(config.inDir);
        File[] files = inDir.listFiles(this::fileFilterAccept);
        Objects.requireNonNull(files);
        if (log.debug) log.debug("Found #&1 files", files.length);
        try {
           listFilesInDirectory(set, inDir, files);
        } catch (IOException e) {
            throw new PermanentFailureException(e, "Error while listing directory [&1]", config.inDir);
        }
        return new ArrayList<>(set);
    }

   private void listFilesInDirectory(SortedSet<String> set, File inDir, File[] files) throws IOException {
      if (log.debug) log.debug("Reading content of directory [&1]...", inDir.getCanonicalPath());
      for (File file : files) {
          if (log.debug) log.debug("Found file [&1]; adding to message list...", file.getName());
          set.add(file.getName());
      }
   }

   @Override
    public Message nativeRead() throws ReaderFailureException {
        if (log.debug) log.debug("Reading file [&1]...", fileName);
        Path path = Paths.get(config.inDir, fileName);
        Message msg = new Message(fileName, fileName);
        try {
            msg.setData(Arrays.toString(Files.readAllBytes(path)));
            return msg;
        } catch (IOException e) {
            throw new TemporaryFailureException(e, "Cannot read the file [&1]", fileName);
        }
    }

    @Override
    public void nativeDelete() throws ReaderFailureException {
        if (log.debug) log.debug("Deleting file [&1]...", fileName);
        File file = new File(config.inDir, fileName);
        deleteFile(file);
    }

    private void deleteFile(File file) throws TemporaryFailureException {
        try {
            if (log.debug) log.debug("Trying to delete the file...");
            IoUtil.delete(file);
        } catch (IOException e) {
            if (file.exists()) throw new TemporaryFailureException(e, "File [&1] still exists", fileName);
            else if (log.info) log.info("File [&1] does not exist - nothing to delete...", fileName);
        }
    }

    @Override
    public void nativeClose() throws PermanentFailureException {
        // close all resources open by nativeInit() here
    }

    private boolean fileFilterAccept(File file) {
        if (file == null || !file.isFile()) {
            if (log.debug) log.debug("File [&1] is null or not a file", file);
            return false;
        }
        boolean doAccept = selectMessage(file.getName());
        if (log.debug) log.debug("File [&1] &2satisfies filter", file, doAccept ? "" : "NOT ");
        return doAccept;
    }
}
